/*
 * Copyright (c) 1993 Addison-Wesley Publishing Company.
 * All rights reserved.  This source code is provided
 * for its instructional value, and is not guaranteed
 * for any particular purpose.
 */
/* mergeaddr.c */
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <netconfig.h>
#include <netdir.h>
#include <tiuser.h>
#include <fcntl.h>

void
main()
{
	int i, fd;
	char *uap, *ruap;
	struct netconfig *ncp;
	struct t_bind *retp;
	struct netbuf nb;
	struct sockaddr_in sa;
	struct nd_mergearg m;

	if ((ncp = getnetconfigent("tcp")) != NULL) {
		/*
		 * If tcp is defined in /etc/netconfig, initialize
		 * the netbuf structure to represent a TCP address.
		 */
		nb.len = sizeof(struct sockaddr_in);
		nb.maxlen = nb.len;
		nb.buf = (char *)&sa;
		sa.sin_family = AF_INET;
		sa.sin_port = 1025;
		sa.sin_addr.s_addr = 0x01020304;
		for (i = 0; i < sizeof(sa.sin_zero); i++)
			sa.sin_zero[i] = 0;

		/*
		 * Next open a transport endpoint and bind the
		 * default address.
		 */
		fd = t_open(ncp->nc_device, O_RDWR, NULL);
		if (fd < 0) {
			t_error("t_open failed");
			exit(1);
		}
		retp = (struct t_bind *)t_alloc(fd, T_BIND, T_ADDR);
		if (retp == NULL) {
			t_error("t_alloc failed");
			exit(1);
		}
		if (t_bind(fd, NULL, retp) < 0) {
			t_error("t_bind failed");
			exit(1);
		}

		/*
		 * Convert the bound address to a universal address
		 * and print it.
		 */
		uap = taddr2uaddr(ncp, &retp->addr);
		if (uap == NULL) {
			netdir_perror("taddr2uaddr failed");
			exit(1);
		}
		printf("local universal address is: %s\n", uap);

		/*
		 * Convert the dummy remote address to a universal
		 * address and print it.
		 */
		ruap = taddr2uaddr(ncp, &nb);
		if (ruap == NULL) {
			netdir_perror("taddr2uaddr failed");
			exit(1);
		}
		printf("remote universal address is: %s\n", ruap);

		/*
		 * Now merge the local address with the remote
		 * address and print the result.
		 */
		m.s_uaddr = uap;
		m.c_uaddr = ruap;
		if (netdir_options(ncp, ND_MERGEADDR, fd,
		  (char *)&m) != 0) {
			netdir_perror("netdir_options failed");
			exit(1);
		}
		printf("merged address is: %s\n", m.m_uaddr);
	}
	exit(0);
}
