/*
 * Copyright (c) 1993 Addison-Wesley Publishing Company.
 * All rights reserved.  This source code is provided
 * for its instructional value, and is not guaranteed
 * for any particular purpose.
 */
#ifdef USEFCNTL

#include <sys/types.h>
#include <sys/file.h>
#include <unistd.h>
#include <fcntl.h>

int
setasync(int fd)
{
	int flags;

	/*
	 * Enable asynchronous mode:
	 * register this process to receive signals
	 * from the socket, get the file flags, and
	 * set the FASYNC flag.
	 */
	if (fcntl(fd, F_SETOWN, getpid()) != -1) {
		flags = fcntl(fd, F_GETFL, 0);
		if (flags != -1)
			return(fcntl(fd, F_SETFL, (flags|FASYNC)));
	}
	return(-1);
}

int
setsync(int fd)
{
	int flags;

	/*
	 * Disable asynchronous mode:
	 * get the file flags, and
	 * clear the FASYNC flag.
	 */
	flags = fcntl(fd, F_GETFL, 0);
	if (flags != -1)
		return(fcntl(fd, F_SETFL, (flags&~FASYNC)));
	return(-1);
}

#else

#include <sys/filio.h>
#include <unistd.h>

int
setasync(int fd)
{
	int flag = 1;
	pid_t pid = getpid();

	/*
	 * Register this process to receive signals
	 * from the socket and enable asynchronous
	 * mode.
	 */
	if (ioctl(fd, FIOSETOWN, &pid) != -1)
		return(ioctl(fd, FIOASYNC, &flag));
	return(-1);
}

int
setsync(int fd)
{
	int flag = 0;

	/*
	 * Disable asynchronous mode.
	 */
	return(ioctl(fd, FIOASYNC, &flag));
}

#endif
